/*
** Module   : ButtonWidget.java
** Abstract : server side button widget implementation
**
** Copyright (c) 2005-2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ----------------Description-----------------
** 001 NVS 20051026   @23482 Created initial version.
** 002 GES 20060114   @23907 Removed totally useless screen-value attr.
** 003 GES 20060803   @28367 Some cleanup and refactoring.
** 004 GES 20061116   @31247 Exposed button-specific flags on the server
**                           side.
** 005 GES 20080320   @37582 Added TYPE attribute.
** 006 SIY 20101001          Added more button options.
** 007 SIY 20101004          Added IMAGE property.
** 008 SIY 20101006          Added LOAD-IMAGE method.
** 009 SIY 20111101          Updated imports.
** 010 CA  20130221          Refactored usage of TYPE attribute.
** 011 CA  20130222          Added NPE protection for cases when a widget is used before is 
**                           attached to a frame (after a CREATE WIDGET statement is used).
** 012 CA  20130927          Added LegacyResource annotation, to determine resource type.
** 013 CA  20131115          Changes related to properly instantiate dynamic resources.
** 014 CA  20140926          Added shared widget configuration support, to allow GUI/ChUI
**                           concrete implementation for the same widget ID.  Refactored the
**                           widget configuration classes: all fields were made public; these
**                           classes need to be as dumb as possible, all logic related to
**                           setting/getting a certain field should be at the widget or at the
**                           caller. Refs #2254
** 015 EVL 20141222          Moving tooltip related methods to the base class to be used for every
**                           widget that needs tooltip feature.
** 016 EVL 20141226          Adding support for methods LOAD-IMAGE-UP, LOAD-IMAGE-DOWN,
**                           LOAD-IMAGE-INSENSITIVE.
** 017 EVL 20150115          Rework for image support implementation.
** 018 EVL 20150128          Button widget now implements image loading interface.
** 019 EVL 20150130          Image phrase handling support added.
*/ 

package com.goldencode.p2j.ui;

import com.goldencode.p2j.util.*;

/**
 * Server side button widget implementation.
 */
@LegacyResource(resource = LegacyResource.BUTTON)
public class ButtonWidget
extends ControlEntity<ButtonConfig>
implements ImageButtonSupport
{
   /**
    * Default constructor.
    */
   public ButtonWidget()
   {
      this(false);
   }
   
   /**
    * Default constructor.
    * 
    * @param   dynamic
    *          Flag indicating if this is a static or dynamic resource.
    */
   public ButtonWidget(boolean dynamic)
   {
      super(dynamic, new ButtonConfig());
   }
   
   /**
    * Set the flag for whether or not this is a default button.
    * 
    * @param    isDefault
    *           <code>true</code> to make it default or <code>false</code>
    *           to make it an ordinary button.
    */
   public void setDefault(boolean isDefault)
   {
      config.isDefault = isDefault;
      pushScreenDefinition();
   }

   /**
    * Set the flag for the AUTO-GO attribute.
    * 
    * @param    autoGo
    *           The new value for the attribute.
    */
   public void setAutoGo(boolean autoGo)
   {
      config.autoGo = autoGo;
      pushScreenDefinition();
   }

   /**
    * Set the flag for the AUTO-ENDKEY attribute.
    * 
    * @param    autoEndKey
    *           The new value for the attribute.
    */
   public void setAutoEndKey(boolean autoEndKey)
   {
      config.autoEndKey = autoEndKey;
      pushScreenDefinition();
   }

   /**
    * Returns the name of the image loaded by LOAD-IMAGE. Corresponds Progress
    * IMAGE attribute
    *
    * @return the name of the image loaded by LOAD-IMAGE. 
    */
   public character getImage()
   {
      return new character(config.imgDef.name);
   }

   /**
    * Set button image file name.
    * 
    * @param   name
    *          Image file name.
    */
   public void setImage(String name)
   {
      loadImage(name);
   }
   
   /**
    * Sets the width for image used to draw button in character units.
    *
    * @param   width
    *          The new value for the image width in characters.
    */
   public void setImageWidthChars(NumberType width)
   {
      if (!width.isUnknown())
      {
         setImageWidthChars(width.intValue());
      }
   }
   
   /**
    * Sets the width for image used to draw button in character units.
    *
    * @param   width
    *          The new value for the image width in characters.
    */
   public void setImageWidthChars(int width)
   {
      config.imgDef.widthChars = width;
      pushScreenDefinition();
      return;
   }
   
   /**
    * Sets the height for image used to draw button in character units.
    *
    * @param   height
    *          The new value for the image height in characters.
    */
   public void setImageHeightChars(NumberType height)
   {
      if (!height.isUnknown())
      {
         setImageHeightChars(height.intValue());
      }
   }
   
   /**
    * Sets the height for image used to draw button in character units.
    *
    * @param   height
    *          The new value for the image height in characters.
    */
   public void setImageHeightChars(int height)
   {
      config.imgDef.heightChars = height;
      pushScreenDefinition();
      return;
   }
   
   /**
    * Sets the width for image used to draw button in pixel units.
    *
    * @param   widthPixels
    *          The new value for the image width in pixels.
    */
   public void setImageWidthPixels(NumberType widthPixels)
   {
      if (!widthPixels.isUnknown())
      {
         setImageWidthPixels(widthPixels.intValue());
      }
   }
   
   /**
    * Sets the width for image used to draw button in pixel units.
    *
    * @param   widthPixels
    *          The new value for the image width in pixels.
    */
   public void setImageWidthPixels(int widthPixels)
   {
      config.imgDef.widthPixels = widthPixels;
      pushScreenDefinition();
      return;
   }
   
   /**
    * Sets the height for image used to draw button in pixel units.
    *
    * @param   heightPixels
    *          The new value for the image height in pixels.
    */
   public void setImageHeightPixels(NumberType heightPixels)
   {
      if (!heightPixels.isUnknown())
      {
         setImageHeightPixels(heightPixels.intValue());
      }
   }

   /**
    * Sets new value of HEIGHT-PIXELS attribute.
    *
    * @param   heightPixels
    *          The new value for the image height in pixels.
    */
   public void setImageHeightPixels(int heightPixels)
   {
      config.imgDef.heightPixels = heightPixels;
      pushScreenDefinition();
      return;
   }

   /**
    * Sets new value of X-offset image phrase option in pixel units.
    *
    * @param   offset
    *          new value of X-offset in pixels.
    */
   public void setImageOffsetX(NumberType offset)
   {
      if (!offset.isUnknown())
      {
         setImageOffsetX(offset.intValue());
      }
   }

   /**
    * Sets new value of X-offset image phrase option in pixel units.
    *
    * @param   offset
    *          new value of X-offset in pixels.
    */
   public void setImageOffsetX(int offset)
   {
      config.imgDef.offsetX = offset;
      pushScreenDefinition();
      return;
   }

   /**
    * Sets new value of Y-offset image phrase option in pixel units.
    *
    * @param   offset
    *          new value of Y-offset in pixels.
    */
   public void setImageOffsetY(NumberType offset)
   {
      if (!offset.isUnknown())
      {
         setImageOffsetY(offset.intValue());
      }
   }

   /**
    * Sets new value of Y-offset image phrase option in pixel units.
    *
    * @param   offset
    *          new value of Y-offset in pixels.
    */
   public void setImageOffsetY(int offset)
   {
      config.imgDef.offsetY = offset;
      pushScreenDefinition();
      return;
   }

   /**
    * Sets new value of ROW-offset image phrase option in character units.
    *
    * @param   offset
    *          new value of ROW-offset in characters.
    */
   public void setImageOffsetRow(NumberType offset)
   {
      if (!offset.isUnknown())
      {
         setImageOffsetRow(offset.intValue());
      }
   }

   /**
    * Sets new value of ROW-offset image phrase option in character units.
    *
    * @param   offset
    *          new value of ROW-offset in characters.
    */
   public void setImageOffsetRow(int offset)
   {
      config.imgDef.offsetRow = offset;
      pushScreenDefinition();
      return;
   }

   /**
    * Sets new value of COLUMN-offset image phrase option in character units.
    *
    * @param   offset
    *          new value of COLUMN-offset in characters.
    */
   public void setImageOffsetColumn(NumberType offset)
   {
      if (!offset.isUnknown())
      {
         setImageOffsetColumn(offset.intValue());
      }
   }

   /**
    * Sets new value of COLUMN-offset image phrase option in character units.
    *
    * @param   offset
    *          new value of COLUMN-offset in characters.
    */
   public void setImageOffsetColumn(int offset)
   {
      config.imgDef.offsetCol = offset;
      pushScreenDefinition();
      return;
   }

   /**
    * Set button up image file name.
    * 
    * @param   name
    *          Image file name.
    */
   public void setImageUp(String name)
   {
      loadImageUp(name);
   }

   /**
    * Sets the width for up image used to draw button in character units.
    *
    * @param   width
    *          The new value for the up image width in characters.
    */
   public void setImageUpWidthChars(NumberType width)
   {
      if (!width.isUnknown())
      {
         setImageUpWidthChars(width.intValue());
      }
   }
   
   /**
    * Sets the width for up image used to draw button in character units.
    *
    * @param   width
    *          The new value for the up image width in characters.
    */
   public void setImageUpWidthChars(int width)
   {
      config.imgUpDef.widthChars = width;
      pushScreenDefinition();
      return;
   }
   
   /**
    * Sets the height for up image used to draw button in character units.
    *
    * @param   height
    *          The new value for the up image height in characters.
    */
   public void setImageUpHeightChars(NumberType height)
   {
      if (!height.isUnknown())
      {
         setImageUpHeightChars(height.intValue());
      }
   }
   
   /**
    * Sets the height for up image used to draw button in character units.
    *
    * @param   height
    *          The new value for the up image height in characters.
    */
   public void setImageUpHeightChars(int height)
   {
      config.imgUpDef.heightChars = height;
      pushScreenDefinition();
      return;
   }
   
   /**
    * Sets the width for up image used to draw button in pixel units.
    *
    * @param   widthPixels
    *          The new value for the up image width in pixels.
    */
   public void setImageUpWidthPixels(NumberType widthPixels)
   {
      if (!widthPixels.isUnknown())
      {
         setImageUpWidthPixels(widthPixels.intValue());
      }
   }
   
   /**
    * Sets the width for up image used to draw button in pixel units.
    *
    * @param   widthPixels
    *          The new value for the up image width in pixels.
    */
   public void setImageUpWidthPixels(int widthPixels)
   {
      config.imgUpDef.widthPixels = widthPixels;
      pushScreenDefinition();
      return;
   }
   
   /**
    * Sets the height for up image used to draw button in pixel units.
    *
    * @param   heightPixels
    *          The new value for the up image height in pixels.
    */
   public void setImageUpHeightPixels(NumberType heightPixels)
   {
      if (!heightPixels.isUnknown())
      {
         setImageUpHeightPixels(heightPixels.intValue());
      }
   }

   /**
    * Sets the height for up image used to draw button in pixel units.
    *
    * @param   heightPixels
    *          The new value for the up image height in pixels.
    */
   public void setImageUpHeightPixels(int heightPixels)
   {
      config.imgUpDef.heightPixels = heightPixels;
      pushScreenDefinition();
      return;
   }

   /**
    * Sets new value of X-offset up image phrase option in pixel units.
    *
    * @param   offset
    *          new value of X-offset in pixels.
    */
   public void setImageUpOffsetX(NumberType offset)
   {
      if (!offset.isUnknown())
      {
         setImageUpOffsetX(offset.intValue());
      }
   }

   /**
    * Sets new value of X-offset up image phrase option in pixel units.
    *
    * @param   offset
    *          new value of X-offset in pixels.
    */
   public void setImageUpOffsetX(int offset)
   {
      config.imgUpDef.offsetX = offset;
      pushScreenDefinition();
      return;
   }

   /**
    * Sets new value of Y-offset up image phrase option in pixel units.
    *
    * @param   offset
    *          new value of Y-offset in pixels.
    */
   public void setImageUpOffsetY(NumberType offset)
   {
      if (!offset.isUnknown())
      {
         setImageUpOffsetY(offset.intValue());
      }
   }

   /**
    * Sets new value of Y-offset up image phrase option in pixel units.
    *
    * @param   offset
    *          new value of Y-offset in pixels.
    */
   public void setImageUpOffsetY(int offset)
   {
      config.imgUpDef.offsetY = offset;
      pushScreenDefinition();
      return;
   }

   /**
    * Sets new value of ROW-offset up image phrase option in character units.
    *
    * @param   offset
    *          new value of ROW-offset in characters.
    */
   public void setImageUpOffsetRow(NumberType offset)
   {
      if (!offset.isUnknown())
      {
         setImageUpOffsetRow(offset.intValue());
      }
   }

   /**
    * Sets new value of ROW-offset up image phrase option in character units.
    *
    * @param   offset
    *          new value of ROW-offset in characters.
    */
   public void setImageUpOffsetRow(int offset)
   {
      config.imgUpDef.offsetRow = offset;
      pushScreenDefinition();
      return;
   }

   /**
    * Sets new value of COLUMN-offset up image phrase option in character units.
    *
    * @param   offset
    *          new value of COLUMN-offset in characters.
    */
   public void setImageUpOffsetColumn(NumberType offset)
   {
      if (!offset.isUnknown())
      {
         setImageUpOffsetColumn(offset.intValue());
      }
   }

   /**
    * Sets new value of COLUMN-offset up image phrase option in character units.
    *
    * @param   offset
    *          new value of COLUMN-offset in characters.
    */
   public void setImageUpOffsetColumn(int offset)
   {
      config.imgUpDef.offsetCol = offset;
      pushScreenDefinition();
      return;
   }

   /**
    * Set button down image file name.
    * 
    * @param   name
    *          Image file name.
    */
   public void setImageDown(String name)
   {
      loadImageDown(name);
   }
   
   /**
    * Sets the width for down image used to draw button in character units.
    *
    * @param   width
    *          The new value for the down image width in characters.
    */
   public void setImageDownWidthChars(NumberType width)
   {
      if (!width.isUnknown())
      {
         setImageDownWidthChars(width.intValue());
      }
   }
   
   /**
    * Sets the width for down image used to draw button in character units.
    *
    * @param   width
    *          The new value for the down image width in characters.
    */
   public void setImageDownWidthChars(int width)
   {
      config.imgDownDef.widthChars = width;
      pushScreenDefinition();
      return;
   }
   
   /**
    * Sets the height for down image used to draw button in character units.
    *
    * @param   height
    *          The new value for the down image height in characters.
    */
   public void setImageDownHeightChars(NumberType height)
   {
      if (!height.isUnknown())
      {
         setImageDownHeightChars(height.intValue());
      }
   }
   
   /**
    * Sets the height for down image used to draw button in character units.
    *
    * @param   height
    *          The new value for the down image height in characters.
    */
   public void setImageDownHeightChars(int height)
   {
      config.imgDownDef.heightChars = height;
      pushScreenDefinition();
      return;
   }
   
   /**
    * Sets the width for down image used to draw button in pixel units.
    *
    * @param   widthPixels
    *          The new value for the down image width in pixels.
    */
   public void setImageDownWidthPixels(NumberType widthPixels)
   {
      if (!widthPixels.isUnknown())
      {
         setImageDownWidthPixels(widthPixels.intValue());
      }
   }
   
   /**
    * Sets the width for down image used to draw button in pixel units.
    *
    * @param   widthPixels
    *          The new value for the down image width in pixels.
    */
   public void setImageDownWidthPixels(int widthPixels)
   {
      config.imgDownDef.widthPixels = widthPixels;
      pushScreenDefinition();
      return;
   }
   
   /**
    * Sets the height for down image used to draw button in pixel units.
    *
    * @param   heightPixels
    *          The new value for the down image height in pixels.
    */
   public void setImageDownHeightPixels(NumberType heightPixels)
   {
      if (!heightPixels.isUnknown())
      {
         setImageDownHeightPixels(heightPixels.intValue());
      }
   }

   /**
    * Sets the height for down image used to draw button in pixel units.
    *
    * @param   heightPixels
    *          The new value for the down image height in pixels.
    */
   public void setImageDownHeightPixels(int heightPixels)
   {
      config.imgDownDef.heightPixels = heightPixels;
      pushScreenDefinition();
      return;
   }

   /**
    * Sets new value of X-offset down image phrase option in pixel units.
    *
    * @param   offset
    *          new value of X-offset in pixels.
    */
   public void setImageDownOffsetX(NumberType offset)
   {
      if (!offset.isUnknown())
      {
         setImageDownOffsetX(offset.intValue());
      }
   }

   /**
    * Sets new value of X-offset down image phrase option in pixel units.
    *
    * @param   offset
    *          new value of X-offset in pixels.
    */
   public void setImageDownOffsetX(int offset)
   {
      config.imgDownDef.offsetX = offset;
      pushScreenDefinition();
      return;
   }

   /**
    * Sets new value of Y-offset down image phrase option in pixel units.
    *
    * @param   offset
    *          new value of Y-offset in pixels.
    */
   public void setImageDownOffsetY(NumberType offset)
   {
      if (!offset.isUnknown())
      {
         setImageDownOffsetY(offset.intValue());
      }
   }

   /**
    * Sets new value of Y-offset down image phrase option in pixel units.
    *
    * @param   offset
    *          new value of Y-offset in pixels.
    */
   public void setImageDownOffsetY(int offset)
   {
      config.imgDownDef.offsetY = offset;
      pushScreenDefinition();
      return;
   }

   /**
    * Sets new value of ROW-offset down image phrase option in character units.
    *
    * @param   offset
    *          new value of ROW-offset in characters.
    */
   public void setImageDownOffsetRow(NumberType offset)
   {
      if (!offset.isUnknown())
      {
         setImageDownOffsetRow(offset.intValue());
      }
   }

   /**
    * Sets new value of ROW-offset down image phrase option in character units.
    *
    * @param   offset
    *          new value of ROW-offset in characters.
    */
   public void setImageDownOffsetRow(int offset)
   {
      config.imgDownDef.offsetRow = offset;
      pushScreenDefinition();
      return;
   }

   /**
    * Sets new value of COLUMN-offset down image phrase option in character units.
    *
    * @param   offset
    *          new value of COLUMN-offset in characters.
    */
   public void setImageDownOffsetColumn(NumberType offset)
   {
      if (!offset.isUnknown())
      {
         setImageDownOffsetColumn(offset.intValue());
      }
   }

   /**
    * Sets new value of COLUMN-offset down image phrase option in character units.
    *
    * @param   offset
    *          new value of COLUMN-offset in characters.
    */
   public void setImageDownOffsetColumn(int offset)
   {
      config.imgDownDef.offsetCol = offset;
      pushScreenDefinition();
      return;
   }

   /**
    * Set button insensitive image file name.
    * 
    * @param   name
    *          Image file name.
    */
   public void setImageIns(String name)
   {
      loadImageInsensitive(name);
   }
   
   /**
    * Sets the width for insensitive image used to draw button in character units.
    *
    * @param   width
    *          The new value for the insensitive image width in characters.
    */
   public void setImageInsWidthChars(NumberType width)
   {
      if (!width.isUnknown())
      {
         setImageInsWidthChars(width.intValue());
      }
   }
   
   /**
    * Sets the width for insensitive image used to draw button in character units.
    *
    * @param   width
    *          The new value for the insensitive image width in characters.
    */
   public void setImageInsWidthChars(int width)
   {
      config.imgInsDef.widthChars = width;
      pushScreenDefinition();
      return;
   }
   
   /**
    * Sets the height for insensitive image used to draw button in character units.
    *
    * @param   height
    *          The new value for the insensitive image height in characters.
    */
   public void setImageInsHeightChars(NumberType height)
   {
      if (!height.isUnknown())
      {
         setImageInsHeightChars(height.intValue());
      }
   }
   
   /**
    * Sets the height for insensitive image used to draw button in character units.
    *
    * @param   height
    *          The new value for the insensitive image height in characters.
    */
   public void setImageInsHeightChars(int height)
   {
      config.imgInsDef.heightChars = height;
      pushScreenDefinition();
      return;
   }
   
   /**
    * Sets the width for insensitive image used to draw button in pixel units.
    *
    * @param   widthPixels
    *          The new value for the insensitive image width in pixels.
    */
   public void setImageInsWidthPixels(NumberType widthPixels)
   {
      if (!widthPixels.isUnknown())
      {
         setImageInsWidthPixels(widthPixels.intValue());
      }
   }
   
   /**
    * Sets the width for insensitive image used to draw button in pixel units.
    *
    * @param   widthPixels
    *          The new value for the insensitive image width in pixels.
    */
   public void setImageInsWidthPixels(int widthPixels)
   {
      config.imgInsDef.widthPixels = widthPixels;
      pushScreenDefinition();
      return;
   }
   
   /**
    * Sets the height for insensitive image used to draw button in pixel units.
    *
    * @param   heightPixels
    *          The new value for the insensitive image height in pixels.
    */
   public void setImageInsHeightPixels(NumberType heightPixels)
   {
      if (!heightPixels.isUnknown())
      {
         setImageInsHeightPixels(heightPixels.intValue());
      }
   }

   /**
    * Sets the height for insensitive image used to draw button in pixel units.
    *
    * @param   heightPixels
    *          The new value for the insensitive image height in pixels.
    */
   public void setImageInsHeightPixels(int heightPixels)
   {
      config.imgInsDef.heightPixels = heightPixels;
      pushScreenDefinition();
      return;
   }

   /**
    * Sets new value of X-offset insensitive image phrase option in pixel units.
    *
    * @param   offset
    *          new value of X-offset in pixels.
    */
   public void setImageInsOffsetX(NumberType offset)
   {
      if (!offset.isUnknown())
      {
         setImageInsOffsetX(offset.intValue());
      }
   }

   /**
    * Sets new value of X-offset insensitive image phrase option in pixel units.
    *
    * @param   offset
    *          new value of X-offset in pixels.
    */
   public void setImageInsOffsetX(int offset)
   {
      config.imgInsDef.offsetX = offset;
      pushScreenDefinition();
      return;
   }

   /**
    * Sets new value of Y-offset insensitive image phrase option in pixel units.
    *
    * @param   offset
    *          new value of Y-offset in pixels.
    */
   public void setImageInsOffsetY(NumberType offset)
   {
      if (!offset.isUnknown())
      {
         setImageInsOffsetY(offset.intValue());
      }
   }

   /**
    * Sets new value of Y-offset insensitive image phrase option in pixel units.
    *
    * @param   offset
    *          new value of Y-offset in pixels.
    */
   public void setImageInsOffsetY(int offset)
   {
      config.imgInsDef.offsetY = offset;
      pushScreenDefinition();
      return;
   }

   /**
    * Sets new value of ROW-offset insensitive image phrase option in character units.
    *
    * @param   offset
    *          new value of ROW-offset in characters.
    */
   public void setImageInsOffsetRow(NumberType offset)
   {
      if (!offset.isUnknown())
      {
         setImageInsOffsetRow(offset.intValue());
      }
   }

   /**
    * Sets new value of ROW-offset insensitive image phrase option in character units.
    *
    * @param   offset
    *          new value of ROW-offset in characters.
    */
   public void setImageInsOffsetRow(int offset)
   {
      config.imgInsDef.offsetRow = offset;
      pushScreenDefinition();
      return;
   }

   /**
    * Sets new value of COLUMN-offset insensitive image phrase option in character units.
    *
    * @param   offset
    *          new value of COLUMN-offset in characters.
    */
   public void setImageInsOffsetColumn(NumberType offset)
   {
      if (!offset.isUnknown())
      {
         setImageInsOffsetColumn(offset.intValue());
      }
   }

   /**
    * Sets new value of COLUMN-offset insensitive image phrase option in character units.
    *
    * @param   offset
    *          new value of COLUMN-offset in characters.
    */
   public void setImageInsOffsetColumn(int offset)
   {
      config.imgInsDef.offsetCol = offset;
      pushScreenDefinition();
      return;
   }

   /**
    * Set NO-FOCUS option.
    * 
    * @param   flag
    *          New value for the option.
    */
   public void setNoFocus(boolean flag)
   {
      config.noFocus = flag;
      pushScreenDefinition();
   }

   /**
    * Set NO-CONVERT-3D-COLORS option.
    * 
    * @param   flag
    *          New value for the option.
    */
   public void setNoConvert3D(boolean flag)
   {
      config.noConvert3D = flag;
      pushScreenDefinition();
   }

   /**
    * Load button image from specified file name.
    * 
    * @param   name
    *          Image file name.
    *          
    * @return  <code>true</code> if operation was successful.
    */
   @Override
   public logical loadImage(String name)
   {
      if (loadImageInt(name, ButtonConfig.IMAGE_UP))
      {
         // only if image was loaded we need to change it's name in config
         config.imgDef.name = name;
         pushScreenDefinition();
         return new logical(true);
      }
      else
      {
         return new logical(false);
      }
   }

   /**
    * Load button image from specified file name.
    * 
    * @param   name
    *          Image file name.
    *          
    * @return  <code>true</code> if operation was successful.
    */
   @Override
   public logical loadImage(character name)
   {
      return name != null && !name.isUnknown() ? loadImage(name.toStringMessage())
                                               : new logical(false);
   }

   /**
    * Load button image for normal state from specified file name.
    * 
    * @param   name
    *          Image file name.
    *          
    * @return  <code>true</code> if operation was successful.
    */
   @Override
   public logical loadImageUp(String name)
   {
      if (loadImageInt(name, ButtonConfig.IMAGE_UP))
      {
         // only if image was loaded we need to change it's name in config
         config.imgUpDef.name = name;
         pushScreenDefinition();
         return new logical(true);
      }
      else
      {
         return new logical(false);
      }
   }

   /**
    * Load button image for normal state from specified file name.
    * 
    * @param   name
    *          Image file name.
    *          
    * @return  <code>true</code> if operation was successful.
    */
   @Override
   public logical loadImageUp(character name)
   {
      return name != null && !name.isUnknown() ? loadImageUp(name.toStringMessage())
                                               : new logical(false);
   }

   /**
    * Load button image for pushed state from specified file name.
    * 
    * @param   name
    *          Image file name.
    *          
    * @return  <code>true</code> if operation was successful.
    */
   @Override
   public logical loadImageDown(String name)
   {
      if (loadImageInt(name, ButtonConfig.IMAGE_DOWN))
      {
         // only if image was loaded we need to change it's name in config
         config.imgDownDef.name = name;
         pushScreenDefinition();
         return new logical(true);
      }
      else
      {
         return new logical(false);
      }
   }

   /**
    * Load button image for pushed state from specified file name.
    * 
    * @param   name
    *          Image file name.
    *          
    * @return  <code>true</code> if operation was successful.
    */
   @Override
   public logical loadImageDown(character name)
   {
      return name != null && !name.isUnknown() ? loadImageDown(name.toStringMessage())
                                               : new logical(false); 
   }

   /**
    * Load button image for disabled state from specified file name.
    * 
    * @param   name
    *          Image file name.
    *          
    * @return  <code>true</code> if operation was successful.
    */
   @Override
   public logical loadImageInsensitive(String name)
   {
      if (loadImageInt(name, ButtonConfig.IMAGE_INS))
      {
         // only if image was loaded we need to change it's name in config
         config.imgInsDef.name = name;
         pushScreenDefinition();
         return new logical(true);
      }
      else
      {
         return new logical(false);
      }
   }

   /**
    * Load button image for disabled state from specified file name.
    * 
    * @param   name
    *          Image file name.
    *          
    * @return  <code>true</code> if operation was successful.
    */
   @Override
   public logical loadImageInsensitive(character name)
   {
      return name != null && !name.isUnknown() ? loadImageInsensitive(name.toStringMessage())
                                               : new logical(false); 
   }

   /**
    * Load image for button.  Internal version.
    * 
    * @param    imageSrc
    *           Image file name to load.
    * @param    type
    *           Image type to load.  Up, Down or Insensitive.
    *          
    * @return  <code>true</code> if operation was successful.
    */
   private boolean loadImageInt(String imageSrc, int type)
   {
      // logical terminal will try to load image from application jar, then ask the
      // client to load from local file system in cas of failure
      return LogicalTerminal.setImage(config.id.getId(), type, imageSrc); 
   }
}
